@@ -3,7 +3,6 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import errno as _errno
 
 from logging import INFO as _INFO
-from logging import WARNING as _WARNING
 from logging import getLogger
 from typing import Optional
 
@@ -22,6 +21,7 @@ _log = getLogger(__name__)
 del getLogger
 
 _R = _hidpp10.REGISTERS
+_IR = _hidpp10.INFO_SUBREGISTERS
 
 #
 #
@@ -107,7 +107,7 @@ class Device(object):
                 self._kind = _hidpp10.DEVICE_KIND[kind]
             else:
                 # Not a notification, force a reading of the wpid
-                pair_info = self.receiver.read_register(_R.receiver_info, 0x20 + number - 1)
+                pair_info = self.receiver.read_register(_R.receiver_info, _IR.pairing_information + number - 1)
                 if pair_info:
                     # may be either a Unifying receiver, or an Unifying-ready
                     # receiver
@@ -123,7 +123,7 @@ class Device(object):
                     kind = self.get_kind_from_index(number, receiver)
                     self._kind = _hidpp10.DEVICE_KIND[kind]
                 else:
-                    # unifying protocol not supported, must be a Nano receiver
+                    # unifying protocol not supported, probably an old Nano receiver
                     device_info = self.receiver.read_register(_R.receiver_info, 0x04)
                     if device_info is None:
                         _log.error('failed to read Nano wpid for device %d of %s', number, receiver)
@@ -151,7 +151,7 @@ class Device(object):
             if self.descriptor is None:
                 # Last chance to correctly identify the device; many Nano
                 # receivers do not support this call.
-                codename = self.receiver.read_register(_R.receiver_info, 0x40 + self.number - 1)
+                codename = self.receiver.read_register(_R.receiver_info, _IR.device_name + self.number - 1)
                 if codename:
                     codename_length = ord(codename[1:2])
                     codename = codename[2:2 + codename_length]
@@ -198,7 +198,7 @@ class Device(object):
             if self.online and self.protocol >= 2.0:
                 self._codename = _hidpp20.get_friendly_name(self)
             elif self.receiver:
-                codename = self.receiver.read_register(_R.receiver_info, 0x40 + self.number - 1)
+                codename = self.receiver.read_register(_R.receiver_info, _IR.device_name + self.number - 1)
                 if codename:
                     codename_length = ord(codename[1:2])
                     codename = codename[2:2 + codename_length]
@@ -231,7 +231,9 @@ class Device(object):
             if self.online and self.protocol >= 2.0:
                 ids = _hidpp20.get_ids(self)
                 if ids:
-                    self._unitId, self._modelId, self._tid_map = _hidpp20.get_ids(self)
+                    self._unitId, self._modelId, self._tid_map = ids
+                    if _log.isEnabledFor(_INFO) and self._serial and self._serial != self._unitId:
+                        _log.info('%s: unitId %s does not match serial %s', self, self._unitId, self._serial)
         return self._modelId
 
     @property
@@ -240,13 +242,16 @@ class Device(object):
             if self.online and self.protocol >= 2.0:
                 ids = _hidpp20.get_ids(self)
                 if ids:
-                    self._unitId, self._modelId, self._tid_map = _hidpp20.get_ids(self)
+                    self._unitId, self._modelId, self._tid_map = ids
+                    if _log.isEnabledFor(_INFO) and self._serial and self._serial != self._unitId:
+                        _log.info('%s: unitId %s does not match serial %s', self, self._unitId, self._serial)
         return self._tid_map
 
     @property
     def kind(self):
         if not self._kind:
-            pair_info = self.receiver.read_register(_R.receiver_info, 0x20 + self.number - 1) if self.receiver else None
+            pair_info = self.receiver.read_register(_R.receiver_info, _IR.pairing_information + self.number - 1) \
+                if self.receiver else None
             if pair_info:
                 kind = ord(pair_info[7:8]) & 0x0F
                 self._kind = _hidpp10.DEVICE_KIND[kind]
@@ -266,7 +271,7 @@ class Device(object):
     @property
     def serial(self):
         if not self._serial and self.receiver:
-            serial = self.receiver.read_register(_R.receiver_info, 0x30 + self.number - 1)
+            serial = self.receiver.read_register(_R.receiver_info, _IR.extended_pairing_information + self.number - 1)
             if serial:
                 ps = ord(serial[9:10]) & 0x0F
                 self._power_switch = _hidpp10.POWER_SWITCH_LOCATION[ps]
@@ -284,7 +289,7 @@ class Device(object):
     @property
     def power_switch_location(self):
         if not self._power_switch and self.receiver:
-            ps = self.receiver.read_register(_R.receiver_info, 0x30 + self.number - 1)
+            ps = self.receiver.read_register(_R.receiver_info, _IR.extended_pairing_information + self.number - 1)
             if ps:
                 ps = ord(ps[9:10]) & 0x0F
                 self._power_switch = _hidpp10.POWER_SWITCH_LOCATION[ps]
@@ -295,7 +300,7 @@ class Device(object):
     @property
     def polling_rate(self):
         if not self._polling_rate and self.receiver:
-            pair_info = self.receiver.read_register(_R.receiver_info, 0x20 + self.number - 1)
+            pair_info = self.receiver.read_register(_R.receiver_info, _IR.pairing_information + self.number - 1)
             if pair_info:
                 self._polling_rate = ord(pair_info[2:3])
             else:
@@ -330,10 +335,9 @@ class Device(object):
 
     @property
     def settings(self):
-        if self._settings is None:
+        if not self._settings:
             self._settings = []
-            if self.descriptor and self.descriptor.settings and self.persister:
-                self._settings = []
+            if self.persister and self.descriptor and self.descriptor.settings:
                 for s in self.descriptor.settings:
                     try:
                         setting = s(self)
@@ -414,8 +418,8 @@ class Device(object):
     def remove_notification_handler(self, id: str):
         """Unregisters the notification handler under name `id`."""
 
-        if id not in self._notification_handlers and _log.isEnabledFor(_WARNING):
-            _log.warn(f'Tried to remove nonexistent notification handler {id} from device {self}.')
+        if id not in self._notification_handlers and _log.isEnabledFor(_INFO):
+            _log.info(f'Tried to remove nonexistent notification handler {id} from device {self}.')
         else:
             del self._notification_handlers[id]
 
@@ -427,14 +431,16 @@ class Device(object):
         return None
 
     def request(self, request_id, *params, no_reply=False):
-        return _base.request(
-            self.handle or self.receiver.handle,
-            self.number,
-            request_id,
-            *params,
-            no_reply=no_reply,
-            long_message=True  # use long messages for all requests - was self.bluetooth
-        )
+
+        if self:
+            return _base.request(
+                self.handle or self.receiver.handle,
+                self.number,
+                request_id,
+                *params,
+                no_reply=no_reply,
+                long_message=self.bluetooth or self.protocol >= 2.0
+            )
 
     def feature_request(self, feature, function=0x00, *params, no_reply=False):
         if self.protocol >= 2.0:
