@@ -23,6 +23,7 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 from logging import DEBUG as _DEBUG
 from logging import ERROR as _ERROR
+from logging import INFO as _INFO
 from logging import WARNING as _WARNING
 from logging import getLogger
 from typing import List
@@ -104,6 +105,7 @@ FEATURE = _NamedInts(
     POINTER_AXIS_ORIENTATION=0x2006,
     VERTICAL_SCROLLING=0x2100,
     SMART_SHIFT=0x2110,
+    SMART_SHIFT_ENHANCED=0x2111,
     HI_RES_SCROLLING=0x2120,
     HIRES_WHEEL=0x2121,
     LOWRES_WHEEL=0x2130,
@@ -113,6 +115,8 @@ FEATURE = _NamedInts(
     POINTER_SPEED=0x2205,
     ANGLE_SNAPPING=0x2230,
     SURFACE_TUNING=0x2240,
+    XY_STATS=0x2250,
+    WHEEL_STATS=0x2251,
     HYBRID_TRACKING=0x2400,
     # Keyboard
     FN_INVERSION=0x40A0,
@@ -160,6 +164,8 @@ FEATURE = _NamedInts(
     SIDETONE=0x8300,
     EQUALIZER=0x8310,
     HEADSET_OUT=0x8320,
+    # Fake features for Solaar internal use
+    MOUSE_GESTURE=0xFE00,
 )
 FEATURE._fallback = lambda x: 'unknown:%04X' % x
 
@@ -521,13 +527,12 @@ class ReprogrammableKeyV4(ReprogrammableKey):
         """
         flags = flags if flags else {}  # See flake8 B006
 
-        if special_keys.MAPPING_FLAG.raw_XY_diverted in flags and flags[special_keys.MAPPING_FLAG.raw_XY_diverted]:
-            # We need diversion to report raw XY, so divert temporarily
-            # (since XY reporting is also temporary)
-            flags[special_keys.MAPPING_FLAG.diverted] = True
-
-        if special_keys.MAPPING_FLAG.diverted in flags and not flags[special_keys.MAPPING_FLAG.diverted]:
-            flags[special_keys.MAPPING_FLAG.raw_XY_diverted] = False
+        # if special_keys.MAPPING_FLAG.raw_XY_diverted in flags and flags[special_keys.MAPPING_FLAG.raw_XY_diverted]:
+        # We need diversion to report raw XY, so divert temporarily
+        # (since XY reporting is also temporary)
+        # flags[special_keys.MAPPING_FLAG.diverted] = True
+        # if special_keys.MAPPING_FLAG.diverted in flags and not flags[special_keys.MAPPING_FLAG.diverted]:
+        # flags[special_keys.MAPPING_FLAG.raw_XY_diverted] = False
 
         # The capability required to set a given reporting flag.
         FLAG_TO_CAPABILITY = {
@@ -1151,7 +1156,7 @@ def decipher_unified_battery(report):
     discharge, level, status, _ignore = _unpack('!BBBB', report[:4])
     status = BATTERY_STATUS[status]
     if _log.isEnabledFor(_DEBUG):
-        _log.debug('battery %s%% charged, level %s, charging %s', discharge, status)
+        _log.debug('battery %s%% charged, level %s, charging %s', discharge, level, status)
     level = (
         _BATTERY_APPROX.full if level == 8  # full
         else _BATTERY_APPROX.good if level == 4  # good
@@ -1312,7 +1317,7 @@ def get_host_names(device):
         if capability_flags & 0x01:  # device can get host names
             for host in range(0, numHosts):
                 hostinfo = feature_request(device, FEATURE.HOSTS_INFO, 0x10, host)
-                _ignore, status, _ignore, numPages, nameLen, _ignore = _unpack('!BBBBBB', hostinfo[:6])
+                _ignore, status, _ignore, _ignore, nameLen, _ignore = _unpack('!BBBBBB', hostinfo[:6])
                 name = ''
                 remaining = nameLen
                 while remaining > 0:
@@ -1323,17 +1328,36 @@ def get_host_names(device):
                     else:
                         remaining = 0
                 host_names[host] = (bool(status), name)
+        # update the current host's name if it doesn't match the system name
+        import socket
+        hostname = socket.gethostname().partition('.')[0]
+        if host_names[currentHost][1] != hostname:
+            set_host_name(device, hostname, host_names[currentHost][1])
+            host_names[currentHost] = (host_names[currentHost][0], hostname)
     return host_names
 
 
-def set_host_name(device, name):
+def set_host_name(device, name, currentName=''):
+    name = bytearray(name, 'utf-8')
+    currentName = bytearray(currentName, 'utf-8')
+    if _log.isEnabledFor(_INFO):
+        _log.info('Setting host name to %s', name)
     state = feature_request(device, FEATURE.HOSTS_INFO, 0x00)
     if state:
-        flags = _unpack('!B', state[:1])[0]
+        flags, _ignore, _ignore, currentHost = _unpack('!BBBB', state[:4])
         if flags & 0x02:
-            hn = name[:min(14, name.find('.'))] if name.find('.') >= 0 else name
-            response = feature_request(device, FEATURE.HOSTS_INFO, 0x40, 0xff, 0, hn)
-            return response
+            hostinfo = feature_request(device, FEATURE.HOSTS_INFO, 0x10, currentHost)
+            _ignore, _ignore, _ignore, _ignore, _ignore, maxNameLen = _unpack('!BBBBBB', hostinfo[:6])
+            if name[:maxNameLen] == currentName[:maxNameLen] and False:
+                return True
+            length = min(maxNameLen, len(name))
+            chunk = 0
+            while chunk < length:
+                response = feature_request(device, FEATURE.HOSTS_INFO, 0x40, currentHost, chunk, name[chunk:chunk + 14])
+                if not response:
+                    return False
+                chunk += 14
+        return True
 
 
 def get_onboard_mode(device):
