@@ -205,7 +205,7 @@ CONTROL = _NamedInts(
     Fn_Down=0x00C0,
     Fn_Up=0x00C1,
     Multiplatform_Lock=0x00C2,
-    App_Switch_Gesture=0x00C3,  # Thumb_Button on MX Master
+    Mouse_Gesture_Button=0x00C3,  # Thumb_Button on MX Master - Logitech name App_Switch_Gesture
     Smart_Shift=0x00C4,  # Top_Button on MX Master
     Microphone=0x00C5,
     Wifi=0x00C6,
