@@ -36,6 +36,7 @@ _file_path = _path.join(_XDG_CONFIG_HOME, 'solaar', 'config.json')
 
 _KEY_VERSION = '_version'
 _KEY_NAME = '_name'
+_KEY_SERIAL = '_serial'
 _KEY_MODEL_ID = '_modelId'
 _KEY_UNIT_ID = '_unitId'
 _configuration = {}
@@ -58,6 +59,7 @@ def _load():
         _log.debug('load => %s', _configuration)
 
     _cleanup(_configuration)
+    _cleanup_load(_configuration)
     _configuration[_KEY_VERSION] = __version__
     return _configuration
 
@@ -98,6 +100,16 @@ def _cleanup(d):
             _cleanup(value)
 
 
+def _cleanup_load(d):
+    # remove boolean values for mouse-gesture and dpi-sliding
+    for device in d.values():
+        if isinstance(device, dict):
+            for setting in ['mouse-gestures', 'dpi-sliding']:
+                mg = device.get(setting, None)
+                if mg is True or mg is False:
+                    del device[setting]
+
+
 class _DeviceEntry(dict):
     def __init__(self, device, **kwargs):
         super(_DeviceEntry, self).__init__(**kwargs)
@@ -114,6 +126,8 @@ class _DeviceEntry(dict):
             self[_KEY_MODEL_ID] = device.modelId
         if device.unitId and device.unitId != self.get(_KEY_UNIT_ID):
             self[_KEY_UNIT_ID] = device.unitId
+        if device.serial and device.serial != '?' and device.serial != self.get(_KEY_SERIAL):
+            self[_KEY_SERIAL] = device.serial
 
     def get_sensitivity(self, name):
         return self.get('_sensitive', {}).get(name, False)
@@ -122,9 +136,16 @@ class _DeviceEntry(dict):
         sensitives = self.get('_sensitive', {})
         if sensitives.get(name) != value:
             sensitives[name] = value
-            self['_sensitive'] = sensitives
+            self.__setitem__('_sensitive', sensitives)
 
 
+# This is neccessarily complicate because the same device can be attached in several different ways.
+# All HID++ 2.0 devices have a modelId and unitId, which can be accessed when they are connected.
+# When paired via a receiver the receiver provides a WPID and a serial number.
+# The unitId and serial number are supposed to be the same, but for some models they are not
+# so even though the modelId includes the WPID it is not always possible to determine the identity of a
+# paired but not receiver-connected device for which the unitId is not known.
+# This only happens is Solaar has never seen the device while it is paired and connected through a receiver.
 def persister(device):
     if not _configuration:
         _load()
@@ -155,6 +176,8 @@ def persister(device):
     if key and not isinstance(entry, _DeviceEntry):
         entry = _DeviceEntry(device, **entry)
         _configuration[key] = entry
+    if isinstance(entry, _DeviceEntry):
+        entry.update(device)
 
     return entry
 
