@@ -28,6 +28,7 @@ from logging import getLogger
 
 from logitech_receiver import Device, Receiver
 from logitech_receiver import base as _base
+from logitech_receiver import hidpp10 as _hidpp10
 from logitech_receiver import listener as _listener
 from logitech_receiver import notifications as _notifications
 from logitech_receiver import status as _status
@@ -76,11 +77,15 @@ class ReceiverListener(_listener.EventsListener):
         assert status_changed_callback
         self.status_changed_callback = status_changed_callback
         _status.attach_to(receiver, self._status_changed)
+        if receiver.isDevice:  # (wired) devices start as active
+            receiver.status.changed(True)
 
     def has_started(self):
         if _log.isEnabledFor(_INFO):
             _log.info('%s: notifications listener has started (%s)', self.receiver, self.receiver.handle)
         notification_flags = self.receiver.enable_connection_notifications()
+        assert self.receiver.isDevice or (notification_flags & _hidpp10.NOTIFICATION_FLAG.wireless), \
+            'Receiver on %s does not support connection notifications, GUI will not show it' % self.receiver.path
         self.receiver.status[_status.KEYS.NOTIFICATION_FLAGS] = notification_flags
         self.receiver.notify_devices()
         self._status_changed(self.receiver)  # , _status.ALERT.NOTIFICATION)
@@ -282,8 +287,6 @@ def _start(device_info):
         rl = ReceiverListener(receiver, _status_callback)
         rl.start()
         _all_listeners[device_info.path] = rl
-        if isDevice:  # (wired) devices start as active
-            receiver.status.changed(True)
         return rl
 
     _log.warn('failed to open %s', device_info)
@@ -392,3 +395,5 @@ def _process_receiver_event(action, device_info):
                     _error_callback('permissions', device_info.path)
             except Exception:
                 _error_callback('permissions', device_info.path)
+        except _base.NoReceiver:
+            _error_callback('nodevice', device_info.path)
