@@ -37,6 +37,7 @@ from . import action as _action
 from . import config_panel as _config_panel
 from . import icons as _icons
 from .about import show_window as _show_about_window
+from .diversion_rules import show_window as _show_diversion_window
 
 _log = getLogger(__name__)
 del getLogger
@@ -327,6 +328,9 @@ def _create_window_layout():
         _('About') + ' ' + NAME, 'help-about', icon_size=_SMALL_BUTTON_ICON_SIZE, clicked=_show_about_window
     )
     bottom_buttons_box.add(about_button)
+    diversion_button = _new_button(_('Rule Editor'), '', icon_size=_SMALL_BUTTON_ICON_SIZE, clicked=_show_diversion_window)
+    bottom_buttons_box.add(diversion_button)
+    bottom_buttons_box.set_child_secondary(diversion_button, True)
 
     # solaar_version = Gtk.Label()
     # solaar_version.set_markup('<small>' + NAME + ' v' + VERSION + '</small>')
@@ -446,7 +450,11 @@ def _device_row(receiver_path, device_number, device=None):
         item = _model.iter_children(receiver_row)
         new_child_index = 0
         while item:
-            assert _model.get_value(item, _COLUMN.PATH) == receiver_path
+            if _model.get_value(item, _COLUMN.PATH) != receiver_path:
+                _log.warn(
+                    'path for device row %s different from path for receiver %s', _model.get_value(item, _COLUMN.PATH),
+                    receiver_path
+                )
             item_number = _model.get_value(item, _COLUMN.NUMBER)
             if item_number == device_number:
                 return item
